summary refs log tree commit diff
path: root/home/.bin/pbd.c
diff options
context:
space:
mode:
authorJune McEnroe <june@causal.agency>2017-07-04 22:10:00 -0400
committerJune McEnroe <june@causal.agency>2017-07-04 22:10:00 -0400
commitb0499d73b55c060b4177c4d8da5d4990be7744dd (patch)
tree92ab70cf72cd9773a48c61d6f0bb7ee12ce1a1d6 /home/.bin/pbd.c
parentReorganize neovim configuration, again (diff)
downloadsrc-b0499d73b55c060b4177c4d8da5d4990be7744dd.tar.gz
src-b0499d73b55c060b4177c4d8da5d4990be7744dd.zip
Consolidate pbd, pbcopy, pbpaste
Diffstat (limited to 'home/.bin/pbd.c')
-rwxr-xr-xhome/.bin/pbd.c55
1 files changed, 53 insertions, 2 deletions
diff --git a/home/.bin/pbd.c b/home/.bin/pbd.c
index e7b61d12..a90974e6 100755
--- a/home/.bin/pbd.c
+++ b/home/.bin/pbd.c
@@ -1,9 +1,13 @@
 #if 0
-exec cc -Wall -Wextra -pedantic $@ -o $(dirname $0)/pbd $0
+cc -Wall -Wextra -pedantic $@ -o $(dirname $0)/pbd $0 && \
+cc -Wall -Wextra -pedantic -DPBCOPY $@ -o $(dirname $0)/pbcopy $0 && \
+exec cc -Wall -Wextra -pedantic -DPBCOPY -DPBPASTE $@ -o $(dirname $0)/pbpaste $0
 #endif
 
 // TCP server which pipes between macOS pbcopy and pbpaste.
 
+#include <arpa/inet.h>
 #include <err.h>
 #include <netinet/in.h>
 #include <stdint.h>
@@ -14,6 +18,8 @@ exec cc -Wall -Wextra -pedantic $@ -o $(dirname $0)/pbd $0
 #include <sysexits.h>
 #include <unistd.h>
 
+#ifndef PBCOPY
+
 static void spawn(const char *cmd, int childFd, int parentFd) {
     pid_t pid = fork();
     if (pid < 0) err(EX_OSERR, "fork");
@@ -71,3 +77,48 @@ int main() {
         if (error) err(EX_IOERR, "close");
     }
 }
+
+#else
+
+int main() {
+    int error;
+
+    int client = socket(PF_INET, SOCK_STREAM, 0);
+    if (client < 0) err(EX_OSERR, "socket");
+
+    struct sockaddr_in addr = {
+        .sin_family = AF_INET,
+        .sin_port = htons(7062),
+        .sin_addr = { .s_addr = htonl(0x7f000001) },
+    };
+
+    error = connect(client, (struct sockaddr *)&addr, sizeof(addr));
+    if (error) err(EX_OSERR, "connect");
+
+#ifdef PBPASTE
+    int fdIn = client;
+    int fdOut = STDOUT_FILENO;
+    error = shutdown(client, SHUT_WR);
+    if (error) err(EX_OSERR, "shutdown");
+#else
+    int fdIn = STDIN_FILENO;
+    int fdOut = client;
+#endif
+
+    char readBuf[4096];
+    ssize_t readLen;
+    while (0 < (readLen = read(fdIn, readBuf, sizeof(readBuf)))) {
+        char *writeBuf = readBuf;
+        ssize_t writeLen;
+        while (0 < (writeLen = write(fdOut, writeBuf, readLen))) {
+            writeBuf += writeLen;
+            readLen -= writeLen;
+        }
+        if (writeLen < 0) err(EX_IOERR, "write");
+    }
+    if (readLen < 0) err(EX_IOERR, "read");
+
+    return EX_OK;
+}
+
+#endif
@@ -1,9 +1,13 @@
 #endif
 
-// TCP server which pipes between macOS pbcopy and pbpaste.
+// TCP server which pipes between macOS pbcopy and pbpaste, and pbcopy and
+// pbpaste implementations which connect to it.
 